Package: duckdbfs
Title: High Performance Remote File System, Database and 'Geospatial'
        Access Using 'duckdb'
Version: 0.1.2
Authors@R: 
  c(person("Carl", "Boettiger", , "cboettig@gmail.com", c("aut", "cre"),
         comment = c(ORCID = "0000-0002-1642-628X")), 
    person("Michael D.","Sumner", role = c("ctb"), email =
    "mdsumner@gmail.com", comment=c(ORCID = "0000-0002-2471-7511")))
Description: Provides friendly wrappers for creating 'duckdb'-backed connections
  to tabular datasets ('csv', parquet, etc) on local or remote file systems.
  This mimics the behaviour of "open_dataset" in the 'arrow' package, 
  but in addition to 'S3' file system also generalizes to any list of 'http' URLs.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/cboettig/duckdbfs,
        https://cboettig.github.io/duckdbfs/
BugReports: https://github.com/cboettig/duckdbfs/issues
Depends: R (>= 4.2)
Imports: DBI, dbplyr, dplyr, duckdb (>= 1.1), fs, glue
Suggests: curl, sf, jsonlite, spelling, minioclient, testthat (>=
        3.0.0)
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2025-10-12 05:58:07 UTC; cboettig
Author: Carl Boettiger [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1642-628X>),
  Michael D. Sumner [ctb] (ORCID:
    <https://orcid.org/0000-0002-2471-7511>)
Maintainer: Carl Boettiger <cboettig@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-12 06:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 17:38:16 UTC; unix
