# Generated by roxygen2: do not edit by hand

export(as_dataset)
export(as_view)
export(cached_connection)
export(close_connection)
export(duckdb_config)
export(duckdb_connect)
export(duckdb_extensions)
export(duckdb_get_config)
export(duckdb_reset)
export(duckdb_s3_config)
export(duckdb_secrets)
export(load_h3)
export(load_spatial)
export(open_dataset)
export(spatial_join)
export(st_read_meta)
export(to_geojson)
export(to_h3j)
export(to_sf)
export(write_dataset)
export(write_geo)
